"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MovingPlatformSystem = exports.MovingPlatformSide = void 0;
const protocol_1 = require("@skeldjs/protocol");
const SystemStatus_1 = require("./SystemStatus");
const net_1 = require("../utils/net");
const events_1 = require("../events");
var MovingPlatformSide;
(function (MovingPlatformSide) {
    MovingPlatformSide[MovingPlatformSide["Right"] = 0] = "Right";
    MovingPlatformSide[MovingPlatformSide["Left"] = 1] = "Left";
})(MovingPlatformSide = exports.MovingPlatformSide || (exports.MovingPlatformSide = {}));
/**
 * Represents a system for doors that must be manually opened.
 *
 * See {@link MovingPlatformSystemEvents} for events to listen to.
 */
class MovingPlatformSystem extends SystemStatus_1.SystemStatus {
    constructor(ship, systemType, data) {
        super(ship, systemType, data);
        this.useId || (this.useId = 0);
        this.target || (this.target = undefined);
        this.side || (this.side = MovingPlatformSide.Right);
    }
    /**
     * The opposite side of the moving platform to move in.
     */
    get oppositeSide() {
        return this.side === MovingPlatformSide.Left
            ? MovingPlatformSide.Right
            : MovingPlatformSide.Left;
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Deserialize(reader, spawn) {
        if (spawn) {
            this.useId = reader.uint8();
            const targetId = reader.uint32();
            this._setTarget(targetId === 255
                ? undefined
                : this.ship.room.getPlayerByNetId(targetId), reader.uint8(), undefined);
        }
        else {
            const newSid = reader.uint8();
            if (net_1.NetworkUtils.seqIdGreaterThan(newSid, this.useId, 1)) {
                this.useId = newSid;
                const targetId = reader.uint32();
                this._setTarget(targetId === 255
                    ? undefined
                    : this.ship.room.getPlayerByNetId(targetId), reader.uint8(), undefined);
            }
        }
    }
    /* eslint-disable-next-line @typescript-eslint/no-unused-vars */
    Serialize(writer, spawn) {
        var _a, _b, _c;
        this.useId++;
        if (this.useId > 255)
            this.useId = 0;
        writer.uint8(this.useId);
        writer.uint32((_c = (_b = (_a = this.target) === null || _a === void 0 ? void 0 : _a.control) === null || _b === void 0 ? void 0 : _b.netId) !== null && _c !== void 0 ? _c : 255);
        writer.uint8(this.side);
        this.dirty = spawn;
    }
    async _setTarget(player, side, rpc) {
        const oldTarget = player;
        const oldSide = this.side;
        this.target = player;
        this.side = side;
        this.dirty = true;
        const ev = await this.emit(new events_1.MovingPlatformPlayerUpdateEvent(this.room, this, rpc, player, side));
        if (ev.reverted) {
            this.target = oldTarget;
            this.side = oldSide;
            this.dirty = true;
            return;
        }
        this.target = ev.alteredPlayer;
        this.side = ev.alteredSide;
        this.dirty = true;
    }
    /**
     * Update the target of the moving platform.
     * @param player The player to set.
     * @param side The direction to move the moving platform in.
     */
    async setTarget(player, side, sendRpc) {
        var _a;
        const resolved = this.ship.room.resolvePlayer(player);
        if (!resolved)
            return;
        if (this.side === side)
            return;
        const oldSide = this.side;
        await this._setTarget(resolved, side, undefined);
        if (sendRpc && this.side !== oldSide) {
            if ((_a = this.target) === null || _a === void 0 ? void 0 : _a.control) {
                this.ship.room.messageStream.push(new protocol_1.RpcMessage(this.target.control.netId, new protocol_1.UsePlatformMessage));
            }
        }
    }
    /**
     * Get on the moving platform as the client's player.
     */
    async getOn() {
        var _a;
        if (!((_a = this.room.myPlayer) === null || _a === void 0 ? void 0 : _a.control))
            return;
        await this.setTarget(this.room.myPlayer, this.oppositeSide, true);
    }
    setSide(side) {
        if (this.side === side)
            return;
        this.side = side;
        this.dirty = true;
    }
}
exports.MovingPlatformSystem = MovingPlatformSystem;
//# sourceMappingURL=MovingPlatformSystem.js.map